package com.nemrodsoftware.maven;

/*
 * Copyright 2001-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import com.nemrodsoftware.INemrodClient;
import com.nemrodsoftware.NemrodClient;
import com.nemrodsoftware.model.Module;

/**
 * Goal which import resource file from a Nemrod instance
 *
 * @goal import
 * 
 * @phase process-sources
 */
public class ImportMojo
    extends AbstractMojo
{
    /**
     * Destination folder.
     * @parameter expression="${import.workfolder}" default-value="${project.build.outputDirectory}"
     */
    private File workfolder;
    
    /**
     * Login.
     * @parameter expression="${import.apikey}" 
     * @required
     */
    private String apikey;

    /**
     * Url.
     * @parameter expression="${import.url}"  default-value="http://live.nemrod-software.com"
     * @required
     */
    private String url;  
    
    /**
     * Project name.
     * @parameter expression="${import.project}"
     * @required
     */
    private String projectName;        

    /**
     * to avoid exporting empty values
     * @parameter expression="${import.exportEmptyValues}" default-value="true"
     * @required
     */
    private Boolean exportEmptyValues;
    
    /**
     * to export only approved values
     * @parameter expression="${import.exportOnlyApprovedValues}" default-value="false"
     * @required
     */
    private Boolean exportOnlyApprovedValues;
    
    /**
     * the list of supported locales
     * @parameter expression="${import.locales}" 
     * @required
     */    
    private List<String> supportedLocales;

    /**
     * the list of bundle to import
     *
     * @parameter expression="${import.bundles}"
     * @required
     */
    private List<String> bundleNames;

    /**
     * export format
     *
     * @parameter expression="${import.format}" default-value="java"
     * @required
     */
    private String format = "java";        

    public void execute()
        throws MojoExecutionException
    {
        try
        {
            INemrodClient client = new NemrodClient(url, apikey);
            
            getLog().info("Action : import ");
            getLog().info("url="+url);
            getLog().info("project="+projectName);
            getLog().info("format="+format);
            
            for (String bundleName : bundleNames)
            {
                for (String supportedLocale : supportedLocales)
                {
                    String moduleName;
                    String modulePath;
                    
                    if (bundleName.contains("/"))
                    {
                        moduleName = StringUtils.substringAfterLast(bundleName,"/");
                        modulePath = StringUtils.substringBeforeLast(bundleName,"/");
                    }
                    else
                    {
                        moduleName= bundleName;
                        modulePath="";
                    }                        
                    
                    getLog().info("");
                    getLog().info("Import bundle "+moduleName+ " for locale : "+supportedLocale+ "...");
                    
                    execute(client,supportedLocale,moduleName,modulePath);
                }
            }            
        }
        catch (Exception e)
        {
            throw new MojoExecutionException("Unable to import resource bundles",e);
        }
    }
    
    public void execute(INemrodClient client, String localeName, String moduleName, String modulePath) throws IOException
    {
        String exportType = format;
        String workFolder = workfolder.getAbsolutePath();
        
        if (StringUtils.isEmpty(projectName))
                throw new IllegalArgumentException("Project name is not set. Please set nemrod.project property");
        if (StringUtils.isEmpty(localeName))
            throw new IllegalArgumentException("Locale name is not set. Please set nemrod.locale property");
        if (StringUtils.isEmpty(exportType))
            throw new IllegalArgumentException("Export type is not set. Please set nemrod.exportformat property");
        if (StringUtils.isEmpty(moduleName))
            throw new IllegalArgumentException("Module name is not set. Please set nemrod.module property");
        
        Module module = client.findModule(projectName,moduleName, modulePath);
        if (module==null)
        {
            throw new IllegalArgumentException("Module not found on nemrod instance for this project. Please check project configuration.");
        }
        
        File file = client.export(projectName, localeName, module, exportType, exportEmptyValues, exportOnlyApprovedValues, workFolder);
        
        getLog().info("Download file="+file.getAbsolutePath()+" - OK");
        
    }    
}
